// SNF4CGP/ExecutiveProcess.cpp
// Copyright (C) 2009 ARM Research Labs, LLC.
// See www.armresearch.com for more information.

#include "ExecutiveProcess.hpp"

#include "../CodeDweller/faults.hpp"

#include <iostream>

using namespace std;

void ExecutiveProcess::initializeOutput() {
    Output.init(Jobs);
}

void ExecutiveProcess::initializeJobPool() {
    Jobs.init(ConfigInfo, Output);
}

void ExecutiveProcess::initializeWorkerPool() {
    Workers.init();
}

void ExecutiveProcess::shutdownWorkerPool() {
    Workers.stop();
}

void ExecutiveProcess::shutdownJobPool() {
    Jobs.stop();
}

void ExecutiveProcess::shutdownOutput() {
    Output.stop();
}

void ExecutiveProcess::dispatchCommand(Command& C) {                            // Job + Worker + Command; go!
    Job& J = Jobs.grab();
    J.setCommand(C);
    Worker& W = Workers.grab();
    W.doJob(J);
}

ExecutiveProcess::ExecutiveProcess(const string& Version, const string& Config) :   // Simple construction.
  VersionInfo(Version),
  ConfigInfo(Config),
  QuitJobNumber(0) {}

ExecutiveProcess::~ExecutiveProcess() {                                         // Deal with uncerimoneous shutdowns.
}

void ExecutiveProcess::doStartup() {                                            // Startup happens in this sequence.
    initializeJobPool();
    initializeOutput();
    initializeWorkerPool();
}

void ExecutiveProcess::dispatchWakeupCommand() {
    Command C;
    C.Type = Command::WAKE;                                                     // Create the WAKE command to announce
    C.Data = VersionInfo;                                                       // that we are alive (and who we are)
    dispatchCommand(C);                                                         // and dispatch a job with it.
}

void ExecutiveProcess::doProcessing() {                                         // Command processing happens here.
    dispatchWakeupCommand();                                                    // First job is to announce ourselves.
    for(;;) {                                                                   // Then process all jobs but QUIT.
        Command C = Input.getCommand();
        switch(C.Type) {

            case Command::QUIT: {
                QuitJobNumber = C.Number;
                cout << "* SNF4CGP[" << C.Number << "] Received QUIT, shutting down..." << endl;
                cout.flush();
                return;
            }

            case Command::NOTGOOD: {
                cout << "* SNF4CGP Input stream has gone bad, shutting down..." << endl;
                cout.flush();
                return;
            }

            default: {
                dispatchCommand(C);
            }
        }
    }
}

void ExecutiveProcess::doShutdown() {                                           // Shutdown happens in this sequence.
    shutdownWorkerPool();
    shutdownOutput();
    shutdownJobPool();
    cout << "* SNF4CGP Shutdown completed." << endl;
    if(0 < QuitJobNumber) {
        cout << QuitJobNumber << " OK" << endl;
    }
    cout.flush();
}
